;; -*- mode: scheme; coding: utf-8 -*-

;;;; Copyright (C) 2011, 2012, 2013
;;;; Free Software Foundation, Inc.
;;;;
;;;; This library is free software; you can redistribute it and/or
;;;; modify it under the terms of the GNU General Public
;;;; License as published by the Free Software Foundation; either
;;;; version 3 of the License, or (at your option) any later version.
;;;; 
;;;; This library is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;;; General Public License for more details.
;;;; 
;;;; You should have received a copy of the GNU General Public License
;;;; along with this library; if not, write to the Free Software
;;;; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
;;;; 02110-1301 USA
;;;;

;;; Commentary:

;;; Code:

(define-module (system regexp)
  ;; guile
  :use-module (ice-9 format)
  :use-module (ice-9 receive)
  :use-module (ice-9 regex)
  ;;:use-module (ice-9 popen)
  ;;:use-module (ice-9 rdelim)

  ;; common
  :use-module (macros reexport)
  :use-module (macros do)

  :export (regexp/show-match
	   regexp/get-regexp-str
	   regexp/get-regexp
	   regexp/get-match
	   regexp/process-match))


(eval-when (compile load eval)
  (re-export-public-interface (ice-9 format)
			      (ice-9 receive)
			      (ice-9 regex)))


;;;
;;; API
;;;

(define (regexp/show-match m)
  (and m
       (format #t "~S~%" m)
       (if (pair? m)
	   (dotimes (i (match:count (cdr m)))
	     (format #t "~A:  ~S~%" i (match:substring (cdr m) i)))
	   (dotimes (i (match:count m))
	     (format #t "~A:  ~S~%" i (match:substring m i))))
       m))

;; Notes:
;;   filters: ((id . (regexp-str . regexp))+)

(define (regexp/get-regexp-str what filters)
  (let ((pair (assoc what filters)))
    (and pair (car (cdr pair)))))

(define (regexp/get-regexp what filters)
  (let ((pair (assoc what filters)))
    (and pair (cddr pair))))

(define (regexp/get-match entry filters)
  ;; entry:   string [an end-user entry]
  (catch 'exit
    (lambda ()
      (dolist (filter filters #f)
	(let* ((what (car filter))
	       (regexp (regexp/get-regexp what filters))
	       (matched? (regexp-exec regexp entry)))
	  (if matched? (throw 'exit (cons what matched?))))))
    (lambda (key match)
      match)))

(define (regexp/process-match match hows)
  ;; (regexp/show-match match)
  (let ((result (list)))
    (for-each (lambda (how)
		;; how: (converter (match-offsets+))
		(let ((convert (car how))
		      (i-result (list)))
		  (for-each (lambda (offset)
			      (set! i-result
				    (cons (convert (match:substring match offset)) i-result)))
		      (cdr how))
		  (set! result
			(cons (reverse! i-result) result))))
	hows)
    (reverse! result)))


#!

(use-modules (system regexp))
(reload-module (resolve-module '(system regexp)))


!#
